import java.util.Vector;
import java.awt.Dimension;

abstract class Shape {
  abstract public void draw();
  abstract public Dimension getBoundingBox(Dimension topCorner);
  abstract public String getInfo();
  abstract public String getName();

  static public void drawLine(int x0, int y0, int x1, int y1) {
  }
}

class Line extends Shape {
  protected int x0, y0, x1, y1;

  public Line(int x0, int y0, int x1, int y1) {
    this.x0 = x0;  this.y0 = y0;
    this.x1 = x1;  this.y1 = y1;
  }

  public void draw() {
    drawLine(x0, y0, x1, y1);
  }

  public Dimension getBoundingBox(Dimension topCorner) {
    topCorner.width = x0;
    topCorner.height = y0;
    return new Dimension(x0, y0);
  }

  public String getName() {
    return "Line";
  }

  public String getInfo() {
    return getName() + " - x0: " + x0  + " y0: " + y0 + " x1: " + x1 + " y1: " + y1;
  }
}


class Circle extends Shape {
  protected int x0, y0, radius;

  public Circle(int x0, int y0, int radius) {
    this.x0 = x0;  this.y0 = y0;
    this.radius = radius;
  }

  public void draw() {
    // Left out for now
  }

  public Dimension getBoundingBox(Dimension topCorner) {
    topCorner.width = x0 + radius;
    topCorner.height = y0 + radius;
    return new Dimension(x0 - radius, y0 - radius);
  }

  public String getName() {
    return "Circle";
  }


  public String getInfo() {
    return getName() +  " - x0: " + x0  + " y0: " + y0 + " radius: " + radius;
  }
}


class Rectangle extends Line {
  public Rectangle(int x0, int y0, int x1, int y1) {
    super(x0, y0, x1, y1);
  }

  public void draw() {
    drawLine(x0, y0, x0, y1);
    drawLine(x0, y1, x1, y1);
    drawLine(x0, y0, x1, y0);
    drawLine(x1, y0, x1, y1);
  }

  public String getName() {
    return "Rectangle";
  }
}


class Draw {
  public static void main(String argv[]) {
    Vector shapes = new Vector();

    shapes.addElement(new Line(100, 200, 200, 50));
    shapes.addElement(new Line(200, 50, 400, 450));
    shapes.addElement(new Circle(350, 350, 35));
    shapes.addElement(new Circle(20, -50, 5));
    shapes.addElement(new Rectangle(0, 0, 1024, 300));

    drawShapes(shapes);
    System.out.println(listShapes(shapes));
  }

  public static void drawShapes(Vector shapes) {
    for (int shapeIndex = 0; shapeIndex < shapes.size(); shapeIndex++) {
      Shape shape = (Shape)shapes.elementAt(shapeIndex);
      shape.draw();
    }
  }

  public static String listShapes(Vector shapes) {
    String list = "";

    for (int shapeIndex = 0; shapeIndex < shapes.size(); shapeIndex++) {
      Shape shape = (Shape)shapes.elementAt(shapeIndex);
      list += shape.getInfo() + "\n";
    }
    return list;
  }
}






