import java.util.Vector;
import java.awt.Dimension;

abstract class Shape {
  abstract public void draw();
  abstract public Dimension getBoundingBox(Dimension topCorner);
  abstract public String getInfo();
  abstract public String getName();

  static public void drawLine(int x0, int y0, int x1, int y1) {
  }
}


class Line extends Shape {
  protected int x0, y0, x1, y1;

  public Line(int x0, int y0, int x1, int y1) {
    this.x0 = x0;  this.y0 = y0;
    this.x1 = x1;  this.y1 = y1;
  }

  public void draw() {
    drawLine(x0, y0, x1, y1);
  }

  public Dimension getBoundingBox(Dimension topCorner) {
    topCorner.width = x0;
    topCorner.height = y0;
    return new Dimension(x0, y0);
  }

  public String getName() {
    return "Line";
  }

  public String getInfo() {
    return getName() + " - x0: " + x0  + " y0: " + y0 + " x1: " + x1 + " y1: " + y1;
  }
}


class Circle extends Shape {
  protected int x0, y0, radius;

  public Circle(int x0, int y0, int radius) {
    this.x0 = x0;  this.y0 = y0;
    this.radius = radius;
  }

  public void draw() {
    // Left out for now
  }

  public Dimension getBoundingBox(Dimension topCorner) {
    topCorner.width = x0 + radius;
    topCorner.height = y0 + radius;
    return new Dimension(x0 - radius, y0 - radius);
  }

  public String getName() {
    return "Circle";
  }


  public String getInfo() {
    return getName() +  " - x0: " + x0  + " y0: " + y0 + " radius: " + radius;
  }
}


class Rectangle extends Line {
  public Rectangle(int x0, int y0, int x1, int y1) {
    super(x0, y0, x1, y1);
  }

  public void draw() {
    drawLine(x0, y0, x0, y1);
    drawLine(x0, y1, x1, y1);
    drawLine(x0, y0, x1, y0);
    drawLine(x1, y0, x1, y1);
  }

  public String getName() {
    return "Rectangle";
  }
}




class Group extends Shape {
  private Vector groupShapes;

  public Group() {
    groupShapes = new Vector();
  }

  public void draw() {
    Draw2.drawShapes(groupShapes);
  }

  public Dimension getBoundingBox(Dimension topCorner) {
    Dimension bottomCorner = new Dimension(),
     sTopCorner = new Dimension(), sBottomCorner;

    for (int shapeIndex = 0; shapeIndex < groupShapes.size(); shapeIndex++) {
      Shape shape = (Shape)groupShapes.elementAt(shapeIndex);

      if (shapeIndex == 0) {
        bottomCorner = shape.getBoundingBox(topCorner);
      } else {
        sBottomCorner = shape.getBoundingBox(sTopCorner);
        if (sBottomCorner.width < bottomCorner.width) bottomCorner.width = sBottomCorner.width;
        if (sBottomCorner.height < bottomCorner.height) bottomCorner.height = sBottomCorner.height;
        if (sTopCorner.width > bottomCorner.width) bottomCorner.width = sBottomCorner.width;
        if (sTopCorner.height > bottomCorner.height) bottomCorner.height = sBottomCorner.height;
      }
    }
    return bottomCorner;
  }

  public String getInfo() {
    return "\n" + getName() + ":\n" + Draw2.listShapes(groupShapes);
  }

  public String getName() {
    return "Group";
  }

  public void addToGroup(Shape shape) {
    groupShapes.addElement(shape);
  }

  public void removeFromGroup(Shape shape) {
    groupShapes.removeElement(shape);
  }
}


class Draw2 {
  public static void main(String argv[]) {
    Vector shapes = new Vector();

    shapes.addElement(new Line(100, 200, 200, 50));
    shapes.addElement(new Circle(350, 350, 35));

    Group group = new Group();
    group.addToGroup(new Line(200, 50, 400, 450));
    group.addToGroup(new Circle(20, -50, 5));
    shapes.addElement(group);

    shapes.addElement(new Rectangle(0, 0, 1024, 300));

    drawShapes(shapes);
    System.out.println(listShapes(shapes));
  }

  public static void drawShapes(Vector shapes) {
    for (int shapeIndex = 0; shapeIndex < shapes.size(); shapeIndex++) {
      Shape shape = (Shape)shapes.elementAt(shapeIndex);
      shape.draw();
    }
  }

  public static String listShapes(Vector shapes) {
    String list = "";

    for (int shapeIndex = 0; shapeIndex < shapes.size(); shapeIndex++) {
      Shape shape = (Shape)shapes.elementAt(shapeIndex);
      list += shape.getInfo() + "\n";
    }
    return list;
  }
}




